@@ -31,7 +31,6 @@ from oslo_serialization import jsonutils
 from oslo_utils import timeutils
 from osprofiler import opts as profiler
 import osprofiler.sqlalchemy
-import six
 import sqlalchemy as sql
 from sqlalchemy.ext import declarative
 from sqlalchemy.orm.attributes import flag_modified, InstrumentedAttribute
@@ -103,8 +102,8 @@ def initialize_decorator(init):
                 if isinstance(attr, InstrumentedAttribute):
                     column = attr.property.columns[0]
                     if isinstance(column.type, String):
-                        if not isinstance(v, six.text_type):
-                            v = six.text_type(v)
+                        if not isinstance(v, str):
+                            v = str(v)
                         if column.type.length and column.type.length < len(v):
                             raise exception.StringLengthExceeded(
                                 string=v, type=k, length=column.type.length)
@@ -112,6 +111,7 @@ def initialize_decorator(init):
         init(self, *args, **kwargs)
     return initialize
 
+
 ModelBase.__init__ = initialize_decorator(ModelBase.__init__)
 
 
@@ -119,6 +119,11 @@ ModelBase.__init__ = initialize_decorator(ModelBase.__init__)
 class JsonBlob(sql_types.TypeDecorator):
 
     impl = sql.Text
+    # NOTE(ralonsoh): set to True as any other TypeDecorator in SQLAlchemy
+    # https://docs.sqlalchemy.org/en/14/core/custom_types.html# \
+    #   sqlalchemy.types.TypeDecorator.cache_ok
+    cache_ok = True
+    """This type is safe to cache."""
 
     def process_bind_param(self, value, dialect):
         return jsonutils.dumps(value)
@@ -144,6 +149,11 @@ class DateTimeInt(sql_types.TypeDecorator):
 
     impl = sql.BigInteger
     epoch = datetime.datetime.fromtimestamp(0, tz=pytz.UTC)
+    # NOTE(ralonsoh): set to True as any other TypeDecorator in SQLAlchemy
+    # https://docs.sqlalchemy.org/en/14/core/custom_types.html# \
+    #   sqlalchemy.types.TypeDecorator.cache_ok
+    cache_ok = True
+    """This type is safe to cache."""
 
     def process_bind_param(self, value, dialect):
         if value is None:
@@ -521,7 +531,7 @@ def handle_conflicts(conflict_type='object'):
                 # exception details out with the raised Conflict exception
                 # as it can contain raw SQL.
                 LOG.debug(_conflict_msg, {'conflict_type': conflict_type,
-                                          'details': six.text_type(e)})
+                                          'details': e})
                 name = None
                 field = None
                 domain_id = None
@@ -560,7 +570,7 @@ def handle_conflicts(conflict_type='object'):
                     # exception details out with the raised Conflict exception
                     # as it can contain raw SQL.
                     LOG.debug(_conflict_msg, {'conflict_type': conflict_type,
-                                              'details': six.text_type(e)})
+                                              'details': e})
                     # NOTE(morganfainberg): This is really a case where the SQL
                     # failed to store the data. This is not something that the
                     # user has done wrong. Example would be a ForeignKey is
