@@ -14,17 +14,10 @@
 
 import abc
 
-import six
-
-import keystone.conf
 from keystone import exception
 
 
-CONF = keystone.conf.CONF
-
-
-@six.add_metaclass(abc.ABCMeta)
-class DomainConfigDriverBase(object):
+class DomainConfigDriverBase(object, metaclass=abc.ABCMeta):
     """Interface description for a Domain Config driver."""
 
     @abc.abstractmethod
