@@ -12,7 +12,7 @@
 
 import uuid
 
-from six.moves import range
+from oslo_config import fixture as config_fixture
 
 from keystone.common import provider_api
 from keystone.credential.providers import fernet as credential_provider
@@ -22,6 +22,7 @@ from keystone.tests.unit import ksfixtures
 from keystone.tests.unit.ksfixtures import database
 
 from keystone.credential.backends import sql as credential_sql
+from keystone import exception
 
 PROVIDERS = provider_api.ProviderAPIs
 
@@ -104,3 +105,11 @@ class SqlCredential(SqlTests):
         # Make sure CredentialModel is handing over a text string
         # to the database. To avoid encoding issues
         self.assertIsInstance(ref.encrypted_blob, str)
+
+    def test_credential_limits(self):
+        config_fixture_ = self.user = self.useFixture(config_fixture.Config())
+        config_fixture_.config(group='credential', user_limit=4)
+        self._create_credential_with_user_id(self.user_foo['id'])
+        self.assertRaises(exception.CredentialLimitExceeded,
+                          self._create_credential_with_user_id,
+                          self.user_foo['id'])
