@@ -12,21 +12,21 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-
 import os
 
 import fixtures
 from migrate.versioning import api as versioning_api
 from migrate.versioning import repository
-from oslo_db.sqlalchemy import test_base
+from oslo_db.sqlalchemy import enginefacade
+from oslo_db.sqlalchemy import test_fixtures as db_fixtures
 from oslo_db.sqlalchemy import test_migrations
+from oslotest import base as test_base
 import sqlalchemy
 import testtools
 
-from keystone.common.sql import contract_repo
-from keystone.common.sql import data_migration_repo
-from keystone.common.sql import expand_repo
-from keystone.common.sql import migrate_repo
+from keystone.common.sql.legacy_migrations import contract_repo
+from keystone.common.sql.legacy_migrations import data_migration_repo
+from keystone.common.sql.legacy_migrations import expand_repo
 from keystone.common.sql import upgrades
 
 
@@ -37,9 +37,8 @@ class DBOperationNotAllowed(Exception):
 class BannedDBSchemaOperations(fixtures.Fixture):
     """Ban some operations for migrations."""
 
-    def __init__(self, banned_ops=None,
-                 migration_repo=migrate_repo.__file__):
-        super(BannedDBSchemaOperations, self).__init__()
+    def __init__(self, banned_ops, migration_repo):
+        super().__init__()
         self._banned_ops = banned_ops or {}
         self._migration_repo = migration_repo
 
@@ -52,7 +51,7 @@ class BannedDBSchemaOperations(fixtures.Fixture):
                 resource_op, repo_name))
 
     def setUp(self):
-        super(BannedDBSchemaOperations, self).setUp()
+        super().setUp()
         explode_lambda = {
             'Table.create': lambda *a, **k: self._explode(
                 'Table.create', self._migration_repo),
@@ -89,7 +88,9 @@ class TestBannedDBSchemaOperations(testtools.TestCase):
         """Test column operations raise DBOperationNotAllowed."""
         column = sqlalchemy.Column()
         with BannedDBSchemaOperations(
-                banned_ops={'Column': ['create', 'alter', 'drop']}):
+            banned_ops={'Column': ['create', 'alter', 'drop']},
+            migration_repo=expand_repo.__file__,
+        ):
             self.assertRaises(DBOperationNotAllowed, column.drop)
             self.assertRaises(DBOperationNotAllowed, column.alter)
             self.assertRaises(DBOperationNotAllowed, column.create)
@@ -98,8 +99,10 @@ class TestBannedDBSchemaOperations(testtools.TestCase):
         """Test table operations raise DBOperationNotAllowed."""
         table = sqlalchemy.Table()
         with BannedDBSchemaOperations(
-                banned_ops={'Table': ['create', 'alter', 'drop',
-                                      'insert', 'update', 'delete']}):
+            banned_ops={'Table': ['create', 'alter', 'drop',
+                                  'insert', 'update', 'delete']},
+            migration_repo=expand_repo.__file__,
+        ):
             self.assertRaises(DBOperationNotAllowed, table.drop)
             self.assertRaises(DBOperationNotAllowed, table.alter)
             self.assertRaises(DBOperationNotAllowed, table.create)
@@ -111,35 +114,19 @@ class TestBannedDBSchemaOperations(testtools.TestCase):
 class KeystoneMigrationsCheckers(test_migrations.WalkVersionsMixin):
     """Walk over and test all sqlalchemy-migrate migrations."""
 
-    # NOTE(xek): We start requiring things be additive in Newton, so
-    # ignore all migrations before the first version in Newton.
-    migrate_file = migrate_repo.__file__
-    first_version = 101
-    # NOTE(henry-nash): We don't ban data modification in the legacy repo,
-    # since there are already migrations that do this for Newton (and these
-    # do not cause us issues, or are already worked around).
-    banned_ops = {'Table': ['alter', 'drop'],
-                  'Column': ['alter', 'drop']}
+    migrate_file = None
+    first_version = 1
+    # A mapping of entity (Table, Column, ...) to operation
+    banned_ops = {}
     exceptions = [
         # NOTE(xek): Reviewers: DO NOT ALLOW THINGS TO BE ADDED HERE UNLESS
         # JUSTIFICATION CAN BE PROVIDED AS TO WHY THIS WILL NOT CAUSE
         # PROBLEMS FOR ROLLING UPGRADES.
-
-        # Migration 102 drops the domain table in the Newton release. All
-        # code that referenced the domain table was removed in the Mitaka
-        # release, hence this migration will not cause problems when
-        # running a mixture of Mitaka and Newton versions of keystone.
-        102,
-
-        # Migration 106 simply allows the password column to be nullable.
-        # This change would not impact a rolling upgrade.
-        106
     ]
 
     @property
     def INIT_VERSION(self):
-        return upgrades.get_init_version(
-            abs_path=os.path.abspath(os.path.dirname(self.migrate_file)))
+        return upgrades.INITIAL_VERSION
 
     @property
     def REPOSITORY(self):
@@ -188,8 +175,7 @@ class KeystoneMigrationsCheckers(test_migrations.WalkVersionsMixin):
         else:
             banned_ops = None
         with BannedDBSchemaOperations(banned_ops, self.migrate_file):
-            super(KeystoneMigrationsCheckers,
-                  self).migrate_up(version, with_data)
+            super().migrate_up(version, with_data)
 
     snake_walk = False
     downgrade = False
@@ -198,23 +184,7 @@ class KeystoneMigrationsCheckers(test_migrations.WalkVersionsMixin):
         self.walk_versions(self.snake_walk, self.downgrade)
 
 
-class TestKeystoneMigrationsMySQL(
-        KeystoneMigrationsCheckers, test_base.MySQLOpportunisticTestCase):
-    pass
-
-
-class TestKeystoneMigrationsPostgreSQL(
-        KeystoneMigrationsCheckers, test_base.PostgreSQLOpportunisticTestCase):
-    pass
-
-
-class TestKeystoneMigrationsSQLite(
-        KeystoneMigrationsCheckers, test_base.DbTestCase):
-    pass
-
-
-class TestKeystoneExpandSchemaMigrations(
-        KeystoneMigrationsCheckers):
+class TestKeystoneExpandSchemaMigrations(KeystoneMigrationsCheckers):
 
     migrate_file = expand_repo.__file__
     first_version = 1
@@ -241,24 +211,40 @@ class TestKeystoneExpandSchemaMigrations(
         # timestamp to datetime and updates the initial value in the contract
         # phase. Adding an exception here to pass expand banned tests,
         # otherwise fails.
-        4
+        4,
+
+        # Migration 79 changes a varchar column length, doesn't
+        # convert the data within that column/table and doesn't rebuild
+        # indexes.
+        79
     ]
 
     def setUp(self):
         super(TestKeystoneExpandSchemaMigrations, self).setUp()
-        self.migrate_fully(migrate_repo.__file__)
 
 
 class TestKeystoneExpandSchemaMigrationsMySQL(
-        TestKeystoneExpandSchemaMigrations,
-        test_base.MySQLOpportunisticTestCase):
-    pass
+        db_fixtures.OpportunisticDBTestMixin,
+        test_base.BaseTestCase,
+        TestKeystoneExpandSchemaMigrations):
+    FIXTURE = db_fixtures.MySQLOpportunisticFixture
+
+    def setUp(self):
+        super(TestKeystoneExpandSchemaMigrationsMySQL, self).setUp()
+        self.engine = enginefacade.writer.get_engine()
+        self.sessionmaker = enginefacade.writer.get_sessionmaker()
 
 
 class TestKeystoneExpandSchemaMigrationsPostgreSQL(
-        TestKeystoneExpandSchemaMigrations,
-        test_base.PostgreSQLOpportunisticTestCase):
-    pass
+        db_fixtures.OpportunisticDBTestMixin,
+        test_base.BaseTestCase,
+        TestKeystoneExpandSchemaMigrations):
+    FIXTURE = db_fixtures.PostgresqlOpportunisticFixture
+
+    def setUp(self):
+        super(TestKeystoneExpandSchemaMigrationsPostgreSQL, self).setUp()
+        self.engine = enginefacade.writer.get_engine()
+        self.sessionmaker = enginefacade.writer.get_sessionmaker()
 
 
 class TestKeystoneDataMigrations(
@@ -286,24 +272,24 @@ class TestKeystoneDataMigrations(
 
     def setUp(self):
         super(TestKeystoneDataMigrations, self).setUp()
-        self.migrate_fully(migrate_repo.__file__)
         self.migrate_fully(expand_repo.__file__)
 
 
 class TestKeystoneDataMigrationsMySQL(
         TestKeystoneDataMigrations,
-        test_base.MySQLOpportunisticTestCase):
-    pass
+        db_fixtures.OpportunisticDBTestMixin):
+    FIXTURE = db_fixtures.MySQLOpportunisticFixture
 
 
 class TestKeystoneDataMigrationsPostgreSQL(
         TestKeystoneDataMigrations,
-        test_base.PostgreSQLOpportunisticTestCase):
-    pass
+        db_fixtures.OpportunisticDBTestMixin):
+    FIXTURE = db_fixtures.PostgresqlOpportunisticFixture
 
 
 class TestKeystoneDataMigrationsSQLite(
-        TestKeystoneDataMigrations, test_base.DbTestCase):
+        TestKeystoneDataMigrations,
+        db_fixtures.OpportunisticDBTestMixin):
     pass
 
 
@@ -346,25 +332,25 @@ class TestKeystoneContractSchemaMigrations(
 
     def setUp(self):
         super(TestKeystoneContractSchemaMigrations, self).setUp()
-        self.migrate_fully(migrate_repo.__file__)
         self.migrate_fully(expand_repo.__file__)
         self.migrate_fully(data_migration_repo.__file__)
 
 
 class TestKeystoneContractSchemaMigrationsMySQL(
         TestKeystoneContractSchemaMigrations,
-        test_base.MySQLOpportunisticTestCase):
-    pass
+        db_fixtures.OpportunisticDBTestMixin):
+    FIXTURE = db_fixtures.MySQLOpportunisticFixture
 
 
 class TestKeystoneContractSchemaMigrationsPostgreSQL(
         TestKeystoneContractSchemaMigrations,
-        test_base.PostgreSQLOpportunisticTestCase):
-    pass
+        db_fixtures.OpportunisticDBTestMixin):
+    FIXTURE = db_fixtures.PostgresqlOpportunisticFixture
 
 
 class TestKeystoneContractSchemaMigrationsSQLite(
-        TestKeystoneContractSchemaMigrations, test_base.DbTestCase):
+        TestKeystoneContractSchemaMigrations,
+        db_fixtures.OpportunisticDBTestMixin):
     # In Sqlite an alter will appear as a create, so if we check for creates
     # we will get false positives.
     def setUp(self):
